@@ -1,8 +1,15 @@
 # Run help(DNFPOptim) for description and argument details.
-DNFOptim <- function(dynamics, data, N = 50, K = 20, R = 1, grids = "Default",
+DNFOptim <- function(dynamics, ...) UseMethod("DNFOptim") 
+
+DNFOptim.default <- function(dynamics, ...){
+  stop("This class of state-space models is not supported by the DNFOptim function.")
+}
+
+DNFOptim.dynamicsSVM <- function(dynamics, data, N = 50, K = 20, R = 1, grids = "Default",
                      rho = 0, delta = 0, alpha = 0, rho_z = 0, nu = 0, jump_params_list = "dummy",
                      ...) {
   model <- dynamics$model
+  
   # Check if custom model has repeated arguments
   if(model == 'Custom'){
     mu_y <- dynamics$mu_y; sigma_y <- dynamics$sigma_y
@@ -48,19 +55,20 @@ DNFOptim <- function(dynamics, data, N = 50, K = 20, R = 1, grids = "Default",
     args_vec <- args_vec[-which(args_vec == "dummy")]
     args_vec <- args_vec[-which(args_vec == "x")]
   }
-   MLE_f <- function(params) {
+  MLE_f <- function(params) {
     # Model from Duffie, Pan, and Singleton (2000):
     if (model == "DuffiePanSingleton") {
       mu <- params[1]
-      kappa <- params[2]
-      theta <- params[3]
-      sigma <- params[4]
-      rho <- params[5]
+      alpha <- params[2]
+      delta <- params[3]
+      rho_z <- params[4]
+      nu <- params[5]
       omega <- params[6]
-      delta <- params[7]
-      alpha <- params[8]
-      rho_z <- params[9]
-      nu <- params[10]
+      kappa <- params[7]
+      theta <- params[8]
+      sigma <- params[9]
+      rho <- params[10]
+
       
       dynamics$mu_y_params <- list(mu, alpha, delta, rho_z, nu, omega)
       dynamics$mu_x_params <- list(kappa, theta)
@@ -77,13 +85,13 @@ DNFOptim <- function(dynamics, data, N = 50, K = 20, R = 1, grids = "Default",
     # Model from Bates (1996):
     else if (model == "Bates") {
       mu <- params[1]
-      kappa <- params[2]
-      theta <- params[3]
-      sigma <- params[4]
-      rho <- params[5]
-      omega <- params[6]
-      delta <- params[7]
-      alpha <- params[8]
+      alpha <- params[2]
+      delta <- params[3]
+      omega <- params[4]
+      kappa <- params[5]
+      theta <- params[6]
+      sigma <- params[7]
+      rho <- params[8]
       
       dynamics$mu_y_params <- list(mu, alpha, delta, omega)
       dynamics$mu_x_params <- list(kappa, theta)
@@ -95,7 +103,7 @@ DNFOptim <- function(dynamics, data, N = 50, K = 20, R = 1, grids = "Default",
       dynamics$delta <- delta
       dynamics$alpha <- alpha
     }
-
+    
     # Model from Heston (1993):
     else if (model == "Heston") {
       mu <- params[1]
@@ -108,21 +116,21 @@ DNFOptim <- function(dynamics, data, N = 50, K = 20, R = 1, grids = "Default",
       dynamics$mu_x_params <- list(kappa, theta)
       dynamics$sigma_x_params <- list(sigma)
       dynamics$rho <- rho
-  
+      
       
     }
-
+    
     # Model from Pitt, Malik, and Doucet (2014)
     else if (model == "PittMalikDoucet") {
       phi <- params[1]
       theta <- params[2]
       sigma <- params[3]
       rho <- params[4]
-      p <- params[5]
-      delta <- params[6]
-      alpha <- params[7]
+      delta <- params[5]
+      alpha <- params[6]
+      p <- params[7]
       
-      dynamics$mu_x_params <- list(theta, phi)
+      dynamics$mu_x_params <- list(phi, theta)
       dynamics$sigma_x_params <- list(sigma)
       
       # Jumps for the PMD Model
@@ -132,7 +140,7 @@ DNFOptim <- function(dynamics, data, N = 50, K = 20, R = 1, grids = "Default",
       dynamics$rho <- rho
       
     }
-
+    
     # Model of  Taylor (1986) with leverage effect
     else if  (model == "TaylorWithLeverage") {
       phi <- params[1]
@@ -140,18 +148,18 @@ DNFOptim <- function(dynamics, data, N = 50, K = 20, R = 1, grids = "Default",
       sigma <- params[3]
       rho <- params[4]
       
-      dynamics$mu_x_params <- list(theta, phi)
+      dynamics$mu_x_params <- list(phi, theta)
       dynamics$sigma_x_params <- list(sigma)
       dynamics$rho <- rho
     }
-
+    
     # Model of  Taylor (1986)
     else if (model == "Taylor") {
       phi <- params[1]
       theta <- params[2]
       sigma <- params[3]
-
-      dynamics$mu_x_params <- list(theta, phi)
+      
+      dynamics$mu_x_params <- list(phi, theta)
       dynamics$sigma_x_params <- list(sigma)
     }
     else if (model == "Custom") {
@@ -242,52 +250,52 @@ DNFOptim <- function(dynamics, data, N = 50, K = 20, R = 1, grids = "Default",
         dynamics$jump_params <- as.list(params)
       }
     }
-
-     if(dynamics$delta < 0 |  dynamics$rho <= -1 |  dynamics$rho >= 1 |  dynamics$nu < 0){
-       LL <- -9999999999999999999
-       return(-LL)
-     }
-     if(any(dynamics$model == c("DuffiePanSingleton", "Bates", "Heston"))){
-       if(theta <= 0){ 
-         LL <- -9999999999999999999
-         return(-LL)
-       }
-       if(kappa <= 0){ 
-         LL <- -9999999999999999999
-         return(-LL)
-       }
-       if(sigma <= 0){ 
-         LL <- -9999999999999999999
-         return(-LL)
-       }
-       if(dynamics$model != "Heston"){
-         if(omega <= 0){
-           LL <- -9999999999999999999
-           return(-LL)
-         }
-       } 
-     }
-     if(any(dynamics$model == c("PittMalikDoucet", "TaylorWithLeverage", "Taylor"))){
-       if(sigma <= 0){ 
-         LL <- -9999999999999999999
-         return(-LL)
-       }
-       if(dynamics$model == "PittMalikDoucet"){
-         if(p <= 0 || p >= 1){
-           LL <- -9999999999999999999
-           return(-LL)
-         }
-       }
-       if(phi <= 0 || phi >= 1){
-         LL <- -9999999999999999999
-         return(-LL)
-       }
-     }
-     # Creating grid for listed models
-     if (length(grids) == 1) { # Check if users input a custom grid
-       grids <- gridMaker(dynamics, R = R, N = N, K = K)
-       
-     }
+    
+    if(dynamics$delta < 0 |  dynamics$rho <= -1 |  dynamics$rho >= 1 |  dynamics$nu < 0){
+      LL <- -9999999999999999999
+      return(-LL)
+    }
+    if(any(dynamics$model == c("DuffiePanSingleton", "Bates", "Heston"))){
+      if(theta <= 0){ 
+        LL <- -9999999999999999999
+        return(-LL)
+      }
+      if(kappa <= 0){ 
+        LL <- -9999999999999999999
+        return(-LL)
+      }
+      if(sigma <= 0){ 
+        LL <- -9999999999999999999
+        return(-LL)
+      }
+      if(dynamics$model != "Heston"){
+        if(omega <= 0){
+          LL <- -9999999999999999999
+          return(-LL)
+        }
+      } 
+    }
+    if(any(dynamics$model == c("PittMalikDoucet", "TaylorWithLeverage", "Taylor"))){
+      if(sigma <= 0){ 
+        LL <- -9999999999999999999
+        return(-LL)
+      }
+      if(dynamics$model == "PittMalikDoucet"){
+        if(p <= 0 || p >= 1){
+          LL <- -9999999999999999999
+          return(-LL)
+        }
+      }
+      if(phi <= 0 || phi >= 1){
+        LL <- -9999999999999999999
+        return(-LL)
+      }
+    }
+    # Creating grid for listed models
+    if (length(grids) == 1) { # Check if users input a custom grid
+      grids <- gridMaker(dynamics, R = R, N = N, K = K)
+      
+    }
     LL <- DNF(data = data, N = N, K = K, R = R, dynamics = dynamics, grids = grids)$log_likelihood
     if (LL == Inf | LL == -Inf) {
       # There can be issues with unusual parameter combinations
@@ -300,108 +308,109 @@ DNFOptim <- function(dynamics, data, N = 50, K = 20, R = 1, grids = "Default",
   params <- optimResults$par
   # Model from Duffie, Pan, and Singleton (2000):
   if (model == "DuffiePanSingleton") {
-    mu <- params[1]
-    kappa <- params[2]
-    theta <- params[3]
-    sigma <- params[4]
-    rho <- params[5]
-    omega <- params[6]
-    delta <- params[7]
-    alpha <- params[8]
-    rho_z <- params[9]
-    nu <- params[10]
+    mu_MLE <- params[1]
+    alpha_MLE <- params[2]
+    delta_MLE <- params[3]
+    rho_z_MLE <- params[4]
+    nu_MLE <- params[5]
+    omega_MLE <- params[6]
+    kappa_MLE <- params[7]
+    theta_MLE <- params[8]
+    sigma_MLE <- params[9]
+    rho_MLE <- params[10]
     
-    dynamics$mu_y_params <- list(mu, alpha, delta, rho_z, nu, omega)
-    dynamics$mu_x_params <- list(kappa, theta)
-    dynamics$sigma_x_params <- list(sigma)
+    dynamics$mu_y_params <- list(mu_MLE, alpha_MLE, delta_MLE,
+                                 rho_z_MLE, nu_MLE, omega_MLE)
+    dynamics$mu_x_params <- list(kappa_MLE, theta_MLE)
+    dynamics$sigma_x_params <- list(sigma_MLE)
     
     # Jump distribution for the DuffiePanSingleton Model
-    dynamics$jump_params <- c(dynamics$h * omega)
-    dynamics$rho <- rho
-    dynamics$delta <- delta
-    dynamics$alpha <- alpha
-    dynamics$rho_z <- rho_z
-    dynamics$nu <- nu
+    dynamics$jump_params <- c(dynamics$h * omega_MLE)
+    dynamics$rho <- rho_MLE
+    dynamics$delta <- delta_MLE
+    dynamics$alpha <- alpha_MLE
+    dynamics$rho_z <- rho_z_MLE
+    dynamics$nu <- nu_MLE
   }
   # Model from Bates (1996):
   else if (model == "Bates") {
-    mu <- params[1]
-    kappa <- params[2]
-    theta <- params[3]
-    sigma <- params[4]
-    rho <- params[5]
-    omega <- params[6]
-    delta <- params[7]
-    alpha <- params[8]
+    mu_MLE <- params[1]
+    alpha_MLE <- params[2]
+    delta_MLE <- params[3]
+    omega_MLE <- params[4]
+    kappa_MLE <- params[5]
+    theta_MLE <- params[6]
+    sigma_MLE <- params[7]
+    rho_MLE <- params[8]
     
-    dynamics$mu_y_params <- list(mu, alpha, delta, omega)
-    dynamics$mu_x_params <- list(kappa, theta)
-    dynamics$sigma_x_params <- list(sigma)
+    dynamics$mu_y_params <- list(mu_MLE, alpha_MLE, delta_MLE, omega_MLE)
+    dynamics$mu_x_params <- list(kappa_MLE, theta_MLE)
+    dynamics$sigma_x_params <- list(sigma_MLE)
     
     # Jump distribution for the Bates Model
-    dynamics$jump_params <- c(dynamics$h * omega)
-    dynamics$rho <- rho
-    dynamics$delta <- delta
-    dynamics$alpha <- alpha
+    dynamics$jump_params <- c(dynamics$h * omega_MLE)
+    dynamics$rho <- rho_MLE
+    dynamics$delta <- delta_MLE
+    dynamics$alpha <- alpha_MLE
   }
   
   # Model from Heston (1993):
   else if (model == "Heston") {
-    mu <- params[1]
-    kappa <- params[2]
-    theta <- params[3]
-    sigma <- params[4]
-    rho <- params[5]
+    mu_MLE <- params[1]
+    kappa_MLE <- params[2]
+    theta_MLE <- params[3]
+    sigma_MLE <- params[4]
+    rho_MLE <- params[5]
     
-    dynamics$mu_y_params <- list(mu)
-    dynamics$mu_x_params <- list(kappa, theta)
-    dynamics$sigma_x_params <- list(sigma)
-    dynamics$rho <- rho
+    dynamics$mu_y_params <- list(mu_MLE)
+    dynamics$mu_x_params <- list(kappa_MLE, theta_MLE)
+    dynamics$sigma_x_params <- list(sigma_MLE)
+    dynamics$rho <- rho_MLE
     
     
   }
   
   # Model from Pitt, Malik, and Doucet (2014)
   else if (model == "PittMalikDoucet") {
-    phi <- params[1]
-    theta <- params[2]
-    sigma <- params[3]
-    rho <- params[4]
-    p <- params[5]
-    delta <- params[6]
-    alpha <- params[7]
+    phi_MLE <- params[1]
+    theta_MLE <- params[2]
+    sigma_MLE <- params[3]
+    rho_MLE <- params[4]
+    delta_MLE <- params[5]
+    alpha_MLE <- params[6]
+    p_MLE <- params[7]
     
-    dynamics$mu_x_params <- list(theta, phi)
-    dynamics$sigma_x_params <- list(sigma)
+    dynamics$mu_x_params <- list(phi_MLE, theta_MLE)
+    dynamics$sigma_x_params <- list(sigma_MLE)
     
     # Jumps for the PMD Model
-    dynamics$jump_params <- c(1, p) # size = 1, p=p for dbinom
-    dynamics$delta <- delta
-    dynamics$alpha <- alpha
-    dynamics$rho <- rho
+    dynamics$jump_params <- c(1, p_MLE) # size = 1, p=p for dbinom
+    dynamics$delta <- delta_MLE
+    dynamics$alpha <- alpha_MLE
+    dynamics$rho <- rho_MLE
     
   }
   
   # Model of  Taylor (1986) with leverage effect
   else if  (model == "TaylorWithLeverage") {
-    phi <- params[1]
-    theta <- params[2]
-    sigma <- params[3]
-    rho <- params[4]
+    phi_MLE <- params[1]
+    theta_MLE <- params[2]
+    sigma_MLE <- params[3]
+    rho_MLE <- params[4]
     
-    dynamics$mu_x_params <- list(theta, phi)
-    dynamics$sigma_x_params <- list(sigma)
-    dynamics$rho <- rho
+    dynamics$mu_x_params <- list(phi_MLE, theta_MLE)
+    dynamics$sigma_x_params <- list(sigma_MLE)
+    dynamics$rho <- rho_MLE
   }
   
   # Model of  Taylor (1986)
   else if (model == "Taylor") {
-    phi <- params[1]
-    theta <- params[2]
-    sigma <- params[3]
+    phi_MLE <- params[1]
+    theta_MLE <- params[2]
+    sigma_MLE <- params[3]
     
-    dynamics$mu_x_params <- list(theta, phi)
-    dynamics$sigma_x_params <- list(sigma)
+    dynamics$mu_x_params <- list(phi_MLE, theta_MLE)
+    dynamics$sigma_x_params <- list(sigma_MLE)
   }
   else if (model == "Custom") {
     # Checking if arguments are repeated in the the drift/diffusion/jumps functions:
@@ -492,5 +501,121 @@ DNFOptim <- function(dynamics, data, N = 50, K = 20, R = 1, grids = "Default",
     }
   }
   DNF_MLE <- DNF(data = data, N = N, K = K, R = R, dynamics = dynamics, grids = grids)
-  return(list(optim = optimResults, SVDNF = DNF_MLE))
+  results <- list(optim = optimResults, SVDNF = DNF_MLE, 
+             # Returns which variables are fixed and which are free (set to "var"):
+             rho = rho, delta = delta, alpha = alpha,
+             rho_z = rho_z, nu = nu, jump_params_list = jump_params_list)
+  class(results) = "DNFOptim"
+  return(results)
+}
+
+print.DNFOptim <- function(x, digits = max(3, getOption("digits") - 3), ...){
+  optim <- x$optim
+  
+  SVDNF <- x$SVDNF
+  dynamics <- SVDNF$dynamics
+  model <- dynamics$model
+  
+  
+  rho <- x$rho
+  alpha <- x$alpha
+  delta <- x$delta
+  nu <- x$nu
+  rho_z <- x$rho_z
+  
+  if (model == "DuffiePanSingleton") {
+    rho <- 'var'
+    alpha <- 'var'
+    delta <- 'var'
+    nu <- 'var'
+    rho_z <- 'var'
+  }
+  # Model from Bates (1996):
+  else if (model == "Bates") {
+    rho <- 'var'
+    alpha <- 'var'
+    delta <- 'var'
+  }
+  
+  # Model from Heston (1993):
+  else if (model == "Heston") {
+    rho <- 'var'
+  }
+  
+  # Model from Pitt, Malik, and Doucet (2014)
+  else if (model == "PittMalikDoucet") {
+    rho <- 'var'
+    alpha <- 'var'
+    delta <- 'var'
+    
+  }
+  
+  # Model of  Taylor (1986) with leverage effect
+  else if  (model == "TaylorWithLeverage") {
+    rho <- 'var'
+    
+  }
+  
+  jump_params_list <- x$jump_params_list
+  
+  # Making a list of the parameter names in the optimization: 
+  
+  mu_y <- dynamics$mu_y; sigma_y <- dynamics$sigma_y
+  mu_x <- dynamics$mu_x; sigma_x <- dynamics$sigma_x
+  
+  # Getting the arguments from drift/diffusion functions
+  args_vec <- c(formalArgs(mu_y), formalArgs(sigma_y),
+                formalArgs(mu_x), formalArgs(sigma_x))
+  
+  # Adding possible parameters that aren't in the drift/diffusion
+  # If set to "var", they will be optimized, or else they are fixed at given values.
+  if(rho == 'var'){
+    args_vec <- c(args_vec, 'rho')
+  }
+  else{
+    dynamics$rho = rho
+  }
+  if(delta == 'var'){
+    args_vec <- c(args_vec, 'delta')
+  } 
+  else{
+    dynamics$delta = delta
+  }
+  if(alpha == 'var'){
+    args_vec <- c(args_vec, 'alpha')
+  }
+  else{
+    dynamics$alpha = alpha
+  }
+  if(rho_z == 'var'){
+    args_vec <- c(args_vec, 'rho_z')
+  }
+  else{
+    dynamics$rho_z = rho_z
+  }
+  if(nu == 'var'){
+    args_vec <- c(args_vec, 'nu')
+  }
+  else{
+    dynamics$nu = nu
+  }
+  
+  args_vec <- c(args_vec, jump_params_list)
+  
+  args_vec <- args_vec[-which(args_vec == "dummy")]
+  args_vec <- args_vec[-which(args_vec == "x")]
+  args_vec <- unique(args_vec)
+  
+  
+  tab <- cbind(Estimates = optim$par)
+  dimnames(tab) <- list(args_vec)
+  colnames(tab) <- list("Estimate")
+
+  cat("\nModel:\n")
+  cat(model, '\n')
+  # MLE coefficient estimates
+  cat("\nCoefficients:\n")
+  print(tab, digits = digits)
+  cat("\n")
+  invisible(x)
 }
